package jp.oov.android.docomosimpatcher;

import android.os.Build;
import android.telephony.ServiceState;
import de.robv.android.xposed.IXposedHookLoadPackage;
import de.robv.android.xposed.callbacks.XC_LoadPackage.LoadPackageParam;
import de.robv.android.xposed.XC_MethodHook;

import static de.robv.android.xposed.XposedHelpers.findAndHookMethod;
import static de.robv.android.xposed.XposedHelpers.getObjectField;
import static de.robv.android.xposed.XposedHelpers.getIntField;
import static de.robv.android.xposed.XposedHelpers.setBooleanField;
import static de.robv.android.xposed.XposedHelpers.callMethod;

public class DocomoSimPatcher implements IXposedHookLoadPackage {
    public void handleLoadPackage(final LoadPackageParam lpparam) throws Throwable {
		if (!lpparam.packageName.equals("com.android.providers.telephony"))
			return;

		if (Build.VERSION.SDK_INT >= 18){
			//API 18 ~
			findAndHookMethod(
				"com.android.internal.telephony.ServiceStateTracker",
				lpparam.classLoader,
				"useDataRegStateForDataOnlyDevices",
				new XC_MethodHook() {
					@Override
					protected void beforeHookedMethod(MethodHookParam param) throws Throwable {
						ServiceState mNewSS = (ServiceState) getObjectField(param.thisObject, "mNewSS");
						callMethod(mNewSS, "setVoiceRegState", callMethod(mNewSS, "getDataRegState"));
					}
				}
			);
			findAndHookMethod(
				"com.android.internal.telephony.gsm.GsmServiceStateTracker",
				lpparam.classLoader,
				"pollStateDone",
				new XC_MethodHook() {
					@Override
					protected void beforeHookedMethod(MethodHookParam param) throws Throwable {
						ServiceState mNewSS = (ServiceState) getObjectField(param.thisObject, "mNewSS");
						callMethod(mNewSS, "setEmergencyOnly", false);
					}
				}
			);
		} else {
			//API 15 ~ 17
			findAndHookMethod(
				"com.android.internal.telephony.gsm.GsmServiceStateTracker",
				lpparam.classLoader,
				"pollStateDone",
				new XC_MethodHook() {
					@Override
					protected void beforeHookedMethod(MethodHookParam param) throws Throwable {
						int newGPRSState = getIntField(param.thisObject, "newGPRSState");
						if (newGPRSState == ServiceState.STATE_IN_SERVICE) {
							ServiceState newSS = (ServiceState) getObjectField(param.thisObject, "newSS");
							newSS.setState(newGPRSState);
							setBooleanField(newSS, "mIsEmergencyOnly", false);
							setBooleanField(param.thisObject, "mEmergencyOnly", false);
						}
					}
				}
			);
		}
	}
}
